@@ -5,7 +5,6 @@ import os
 from django.contrib.gis.db import models
 from django.conf import settings
 from django.core.urlresolvers import reverse
-from django.contrib.auth.models import User
 from django.utils.translation import ugettext, ugettext_lazy as _
 from django.core.signing import Signer
 from django.contrib import messages
@@ -129,8 +128,8 @@ class Map(NamedModel):
     )
     modified_at = models.DateTimeField(auto_now=True)
     tilelayer = models.ForeignKey(TileLayer, blank=True, null=True, related_name="maps",  verbose_name=_("background"))
-    owner = models.ForeignKey(User, blank=True, null=True, related_name="owned_maps", verbose_name=_("owner"))
-    editors = models.ManyToManyField(User, blank=True, verbose_name=_("editors"))
+    owner = models.ForeignKey(settings.AUTH_USER_MODEL, blank=True, null=True, related_name="owned_maps", verbose_name=_("owner"))
+    editors = models.ManyToManyField(settings.AUTH_USER_MODEL, blank=True, verbose_name=_("editors"))
     edit_status = models.SmallIntegerField(choices=EDIT_STATUS, default=OWNER, verbose_name=_("edit status"))
     share_status = models.SmallIntegerField(choices=SHARE_STATUS, default=PUBLIC, verbose_name=_("share status"))
     settings = DictField(blank=True, null=True, verbose_name=_("settings"))
